#include "MaskedSprite.h"

MaskedSprite* MaskedSprite::create(const char* textureName, const char* maskedTextureName)
{
    MaskedSprite *pSprite = new MaskedSprite();
    if (pSprite && pSprite->init(textureName, maskedTextureName))
    {
        pSprite->autorelease();
        return pSprite;
    }
    CC_SAFE_DELETE(pSprite);
    return NULL;
}

bool MaskedSprite::init(const char* textureName, const char* maskedTextureName)
{
    if(!CCSprite::initWithFile(textureName))
    {
        return false;
    }
    
    _maskedTexture = CCTextureCache::sharedTextureCache()->addImage(maskedTextureName);
    _maskedTexture->retain();
    
    CCGLProgram* pProgram = new CCGLProgram();
    pProgram->initWithVertexShaderFilename("MaskedSprite.vert", "MaskedSprite.frag");
    
    setShaderProgram(pProgram);
    
    pProgram->addAttribute(kCCAttributeNamePosition, kCCVertexAttrib_Position);
    pProgram->addAttribute(kCCAttributeNameColor, kCCVertexAttrib_Color);
    pProgram->addAttribute(kCCAttributeNameTexCoord, kCCVertexAttrib_TexCoords);
    
    pProgram->link();
    pProgram->updateUniforms();
    
    _locTexture = glGetUniformLocation(pProgram->getProgram(), "u_texture");
    _locMask = glGetUniformLocation(pProgram->getProgram(), "u_mask");
    
    pProgram->release();
    
    return true;
}

void MaskedSprite::draw()
{
    CC_NODE_DRAW_SETUP();
    
    ccGLBlendFunc( m_sBlendFunc.src, m_sBlendFunc.dst );
    
    glActiveTexture(GL_TEXTURE0);
    glBindTexture( GL_TEXTURE_2D,  m_pobTexture->getName() );
    glUniform1i(_locTexture, 0);
    
    glActiveTexture(GL_TEXTURE1);
    glBindTexture( GL_TEXTURE_2D,  _maskedTexture->getName() );
    glUniform1i(_locMask, 1);
    

    ccGLEnableVertexAttribs( kCCVertexAttribFlag_PosColorTex );

    long offset = (long)&m_sQuad;
    
#define kQuadSize sizeof(m_sQuad.bl)
    
    // vertex
    int diff = offsetof( ccV3F_C4B_T2F, vertices);
    glVertexAttribPointer(kCCVertexAttrib_Position, 3, GL_FLOAT, GL_FALSE, kQuadSize, (void*) (offset + diff));
    
    // texCoods
    diff = offsetof( ccV3F_C4B_T2F, texCoords);
    glVertexAttribPointer(kCCVertexAttrib_TexCoords, 2, GL_FLOAT, GL_FALSE, kQuadSize, (void*)(offset + diff));
    
    // color
    diff = offsetof( ccV3F_C4B_T2F, colors);
    glVertexAttribPointer(kCCVertexAttrib_Color, 4, GL_UNSIGNED_BYTE, GL_TRUE, kQuadSize, (void*)(offset + diff));
    
    
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    
    CHECK_GL_ERROR_DEBUG();
}